#!/bin/sh
export PATH='/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/bin/site_perl:/usr/bin/vendor_perl:/usr/bin/core_perl'
echo 'name:                 bloomfilter
version:              2.0.1.0
visibility:           public
id:                   bloomfilter-2.0.1.0-5J1JlJ4Yhn8CauN0mmGZqI
key:                  bloomfilter-2.0.1.0-5J1JlJ4Yhn8CauN0mmGZqI
license:              BSD-3-Clause
maintainer:           Bryan O'\''Sullivan <bos@serpentine.com>
author:               Bryan O'\''Sullivan <bos@serpentine.com>
stability:            provisional
homepage:             https://github.com/bos/bloomfilter
synopsis:             Pure and impure Bloom Filter implementations.
description:          Pure and impure Bloom Filter implementations.
category:             Data
abi:                  7f97c6734a0fd5e274a280f44d873419
exposed:              True
exposed-modules:
    Data.BloomFilter Data.BloomFilter.Easy Data.BloomFilter.Hash
    Data.BloomFilter.Mutable

hidden-modules:
    Data.BloomFilter.Array Data.BloomFilter.Mutable.Internal
    Data.BloomFilter.Util

import-dirs:          /usr/lib/ghc-8.10.5/site-local/bloomfilter-2.0.1.0
library-dirs:         /usr/lib/ghc-8.10.5/site-local/bloomfilter-2.0.1.0
dynamic-library-dirs: /usr/lib
data-dir:             /usr/share/i386-linux-ghc-8.10.5/bloomfilter-2.0.1.0
hs-libraries:         HSbloomfilter-2.0.1.0-5J1JlJ4Yhn8CauN0mmGZqI
include-dirs:         /usr/lib/ghc-8.10.5/site-local/bloomfilter-2.0.1.0/include
includes:             lookup3.h
depends:
    array-0.5.4.0 base-4.14.2.0 bytestring-0.10.12.0 deepseq-1.4.4.0

haddock-interfaces:   /usr/share/doc/haskell-bloomfilter/html/bloomfilter.haddock
haddock-html:         /usr/share/doc/haskell-bloomfilter/html
' | '/usr/bin/ghc-pkg' 'update' '--force' '-' '--global' '--no-user-package-db' "$@"

